#include <stdio.h>
#include <windows.h>

#define FORCEDINLINE __forceinline

void calculate_data(int in, int *out)
{
    // stores five times the input `in' into `out'
    *out = in * 5;
}

FORCEDINLINE void calculate_data_obf(int in, int *out)
{
    // check if any hardware breakpoints were set
    CONTEXT ctx = {CONTEXT_DEBUG_REGISTERS};

    // Debug Register 7 contains the flags (type of breakpoint
    // and amount of bytes), so checking that against nonzero
    // provides enough information for us
    if(GetThreadContext(GetCurrentThread(), &ctx) == FALSE ||
            ctx.Dr7 != 0) {
        printf("No Hardware Breakpoints please ;)\n");
        return;
    }

    // no hardware breakpoints were found, let's
    // continue with the real function.
    calculate_data(in, out);
}

#define calculate_data calculate_data_obf

int main(void)
{
    for (int i = 0, out; i < 25; i++) {
        calculate_data(i, &out);
        printf("%d -> %d\n", i, out);

        // the 15th time we set a hardware breakpoint
        if(i == 15) {
            CONTEXT ctx = {CONTEXT_DEBUG_REGISTERS};
            ctx.Dr0 = (DWORD) &calculate_data;
            ctx.Dr7 = 0x00000001;
            SetThreadContext(GetCurrentThread(), &ctx);
        }
    }
}
