
#define FORCEDINLINE __forceinline

// simple encryption/decryption function which
// xor's the input with 0x42.
FORCEDINLINE void _object_crypt(void *obj, int size)
{
	for (int i = 0; i < size; i++) {
		((unsigned char *) obj)[i] ^= 0x42;
	}
}

FORCEDINLINE void memory_init_obf(memory_t *mem, int maxsize)
{
	memory_init(mem, maxsize);
	// encrypt memory context
	_object_crypt(mem, sizeof(memory_t));
}

FORCEDINLINE void *memory_get_obf(memory_t *mem, int size)
{
	// decrypt memory context
	_object_crypt(mem, sizeof(memory_t));
	// call original function
	void *ret = memory_get(mem, size);
	// encrypt memory context
	_object_crypt(mem, sizeof(memory_t));
	// return the.. return value..
	return ret;
}

FORCEDINLINE void memory_status_obf(memory_t *mem, int *used, int *left)
{
	// decrypt memory context
	_object_crypt(mem, sizeof(memory_t));
	// call original function
	memory_status(mem, used, left);
	// encrypt memory context
	_object_crypt(mem, sizeof(memory_t));
}

FORCEDINLINE void memory_destroy_obf(memory_t *mem)
{
	// decrypt memory context
	_object_crypt(mem, sizeof(memory_t));
	// call original function
	memory_destroy(mem);
	// no need to encrypt, context is no longer
	// valid anyway
}

#define memory_init memory_init_obf
#define memory_get memory_get_obf
#define memory_status memory_status_obf
#define memory_destroy memory_destroy_obf
